---@param label string @Slider label.
---@param value refnumber|number @Current slider value.
---@param min number? @Default value: 0.
---@param max number? @Default value: 1.
---@param format string|'%.3f'|nil @C-style format string. Default value: '%.3f'.
---@param power number? @Power for non-linear slider. Default value: 1 (linear).
---@return number @Possibly updated slider value.
---@return boolean @True if slider has moved.
---@overload fun(label: string, value: number, min: number, max: number, format: string, power: number): number, boolean
function TooltipSlider(label, value, min, max, format, power)
    local ret, changed = ui.slider(label, value, min, max, format, power)
    if ui.itemHovered() then
        ui.tooltip(function()
            ui.text("[Ctrl]+[Click]: write custom value")
        end)
    end
    return ret, changed
end

function SpacingY()
    ui.offsetCursorY(5)
end

function SpacingSeparator()
    ui.offsetCursorY(5)
    ui.separator()
    ui.offsetCursorY(5)
end

function RemoveButton(key)
    if ui.modernButton then
        return ui.modernButton('##Remove' .. key, vec2(22, 22), 0, ui.Icons.Trash)
    end

    return ui.button('Remove##' .. key)
end

---@return boolean result true if paste was pressed, false else
function CopyPaste(id, name, obj, copiedKeys, onPaste)
    if ui.button('Copy##' .. id) then
        local copyObj = {}
        table.forEach(copiedKeys, function(key)
            copyObj[key] = obj[key]
        end)
        ui.setClipboardText(stringify(copyObj))
        ui.toast(ui.Icons.Clipboard, name .. ' copied!')
    end
    ui.sameLine()
    if ui.button('Paste##' .. id) then
        local copyObj = stringify.tryParse(ui.getClipboardText(), {})
        if copyObj and table.nkeys(copyObj) > 0 then
            table.forEach(copiedKeys, function(key)
                obj[key] = copyObj[key]
            end)
            ui.toast(ui.Icons.Clipboard, name .. ' imported!')
            if onPaste then
                onPaste()
            end
        else
            ui.toast(ui.Icons.Clipboard, 'Invalid string. Only paste valid data.')
        end
        return true
    end
    return false
end

-- override due to boolean returned on button click not color change
local originalColorButton = ui.colorButton
---Color button control. Returns true if color has changed (as usual with Lua, colors are passed)
---by reference so update value would be put in place of old one automatically.
---@param label string
---@param color rgbm
---@param flags ui.ColorPickerFlags?
---@param size vec2?
---@return boolean
---@diagnostic disable-next-line: duplicate-set-field
function ui.colorButton(label, color, flags, size)
    local originalColor = color:clone()
    originalColorButton(label, color, flags, size)
    return originalColor:vec4():distance(color:vec4()) > 0
end
