--
-- MapDisplay for Assetto Corsa
-- Author AW939
--

require('helpers/helpers')
require('helpers/ui_helpers')
require('helpers/model/constants')
require('helpers/model/car')
require('widgets/map_legend')
require('widgets/map_progress')
require('widgets/map_full')
require('widgets/map_mini')

ui.setAsynchronousImagesLoading(true)

-- globals
APP_DIR = ac.getFolder(ac.FolderID.ACApps) .. '\\lua\\map_display\\'
CARS = {}
FOCUSED_CAR = {}

local isInitialized = false
local currentSessionIndex = 0
local isSessionStarted = false

-- init widgets

local fullMap, miniMap, progressMap, fullLegend, miniLegend

-- privates
local function initialize()
  CARS = {}
  for i = 0, ac.getSim().carsCount - 1 do
    CARS[i + 1] = Car(i);
  end
  FOCUSED_CAR = CARS[0]

  fullMap = MapFull('MapFull')
  miniMap = MapMini('MapMini')
  progressMap = MapProgress('MapProgress')

  fullLegend = MapLegend('MapDisplay', fullMap.settings)
  miniLegend = MapLegend('MapDisplayMini', miniMap.settings)

  isInitialized = true
  currentSessionIndex = ac.getSim().currentSessionIndex
  isSessionStarted = false
  ac.debug('Initialized', ac.getSim().time)
end

if ac.onSessionStart then
  ac.onSessionStart(function()
    initialize()
  end)
end

-- public widget script functions

-- map main
function script.mapFull(dt)
  fullMap:draw()
  ui.text("") -- otherwise the pinned window is no longer clickable
end

function script.mapFullHide(dt)
  fullMap:hide()
end

function script.mapFullSettings(dt)
  fullMap:drawSettings()
end

-- map mini
function script.mapMini(dt)
  miniMap:draw()
  ui.text("") -- otherwise the pinned window is no longer clickable
end

function script.mapMiniHide(dt)
  miniMap:hide()
end

function script.mapMiniSettings(dt)
  miniMap:drawSettings()
end

-- map progress
function script.mapProgress(dt)
  progressMap:draw()
  ui.text("") -- otherwise the pinned window is no longer clickable
end

function script.mapProgressHide(dt)
  progressMap:hide()
end

function script.mapProgressSettings(dt)
  progressMap:drawSettings()
end

-- legend main
function script.legendMain(dt)
  fullLegend:draw()
end

function script.legendMainHide(dt)
  fullLegend:hide()
end

function script.legendMainSettings(dt)
  fullLegend:drawSettings()
end

-- legend mini
function script.legendMini(dt)
  miniLegend:draw()
end

function script.legendMiniHide(dt)
  miniLegend:hide()
end

function script.legendMiniSettings(dt)
  miniLegend:drawSettings()
end

---@diagnostic disable-next-line: duplicate-set-field
function script.update(dt)
  local sim = ac.getSim()

  if sim.currentSessionIndex ~= currentSessionIndex or sim.isSessionStarted ~= isSessionStarted then
    isInitialized = false
  end

  if not isInitialized then
    initialize()
  end

  if sim.isSessionStarted then
    isSessionStarted = true
  end

  table.forEach(CARS, function(car, index)
    car:update(dt, index, sim)
    if (car.id == ac.getSim().focusedCar) then
      FOCUSED_CAR = car
    end
  end)
  table.sort(CARS, Car.getComparatorForSession())

  fullMap:update(dt)
  miniMap:update(dt)
  progressMap:update(dt)
  fullLegend:update(dt)
  miniLegend:update(dt)
end
