---@diagnostic disable: undefined-field

require('widgets/map_full')
require('widgets/map_mini_settings')

MapMini = class('MapMini', MapFull)
function MapMini:initialize(name) -- constructor
  MapFull.initialize(self, name)
end

function MapMini:initializeSettings()
  self.settings = MapMiniSettings(self.name)
end

function MapMini:getScale(maxSpace, imageSize)
  return (0.5 * self.settings.mapInfo.scaleFactor) *
      (maxSpace / self.settings.mini.visibleRadiusMeters)
end

function MapMini:getImageMargin()
  return 0
end

function MapMini:getImageRotation()
  local cameraLook = ac.getSim().cameraLook
  ---@diagnostic disable-next-line: deprecated
  return math.atan2(cameraLook.x, cameraLook.z) / (math.pi / 180) + 180
end

function MapMini:getImagePositionOffset(imageScale, scaledImageCenter, contentSize, rotation)
  local cameraPosition = ac.getSim().cameraPosition
  local offset = self:transformPointToMap(
        vec2(cameraPosition.x, cameraPosition.z),
        imageScale,
        scaledImageCenter,
        rotation)
      :sub(scaledImageCenter)
      :scale(-1)
  return offset:set(offset.x,
    offset.y + (math.min(contentSize.x, contentSize.y) * (self.settings.mini.verticalPosition - 50) * 0.01)) -- make dynamic: center offset percent
end

function MapMini:startContentClip(p1, p2)
  if self.settings.mini.clippingMode ~= ClippingMode.None then
    ui.pushClipRect(p1, p2)
  end
end

function MapMini:endContentClip(p1, p2)
  if self.settings.mini.clippingMode ~= ClippingMode.None then
    ui.popClipRect()
    if self.settings.mini.clippingMode == ClippingMode.Fade then
      ui.renderTexture({
        filename = 'images/fade_alpha.png',
        p1 = p1,
        -- seems like a bug, for some reason the window position is added to this point
        p2 = p2:sub(ui.windowPos()),
        color = rgbm(1, 1, 1, 1),
        blendMode = render.BlendMode.BlendMultiply
      })
    end
  end
end

function MapMini:drawTeleports(imageScale, imageCenter, imageOffset, rotation)
  -- do nothing, teleport points not displayed in minimap
end

function MapMini:initOnlineTeleportPoints()
  -- do nothing, teleport points not displayed in minimap
end
