---@diagnostic disable: undefined-field

require('widgets/map_full_settings')

MapMiniSettings = class('MapMiniSettings', MapFullSettings)

MapMiniSettings.defaultMini = require('widgets/settings/mini_defaults')
MapMiniSettings.defaultMapGen = require('widgets/settings/map_gen_defaults_mini')

function MapMiniSettings:initialize(name) -- constructor
  MapFullSettings.initialize(self, name)

  self.mini = ac.storage(self.defaultMini(), name .. '_Mini')
end

function MapMiniSettings:initializeMapGenerator()
  self.mapGenStore = ac.storage(self.defaultMapGen(), self.name .. '_MapGeneration')
  self.mapGenerator = MapGenerator('_mini', MapGenerator.SizeMode.Adaptive)
end

function MapMiniSettings:mainContent()
  ui.tabBar('tabs', ui.TabBarFlags.FittingPolicyScroll + ui.TabBarFlags.TabListPopupButton, function()
    ui.tabItem('General', Bind(self.tabMain, self))
    ui.tabItem('Mini Map', Bind(self.tabMini, self))
    ui.tabItem('Track', Bind(self.tabTrack, self))
    ui.tabItem('Drivers', Bind(self.tabDrivers, self))
    ui.tabItem('Map Generation', Bind(self.tabMapGen, self))
    ui.tabItem('Classes', Bind(self.tabClasses, self))
    ui.tabItem('Freeroam', Bind(self.tabFreeroam, self))
    ui.tabItem('Debug', Bind(self.tabDebug, self))
    return nil
  end)
end

function MapMiniSettings:tabMini()
  ui.text("Mini Map")
  SpacingY()

  self.mini.visibleRadiusMeters = TooltipSlider('Visible Radius', self.mini.visibleRadiusMeters, 100, 2000, '%.0f m')
  self.mini.verticalPosition = TooltipSlider('Vertical Position', self.mini.verticalPosition, 0, 100, '%.0f%%')

  local clippingModeTexts = {
    [ClippingMode.None] = 'Show',
    [ClippingMode.Quad] = 'Quad',
    [ClippingMode.Fade] = 'Round Fade',
  }
  local clippingModePreview = clippingModeTexts[self.mini.clippingMode]
  ui.combo('Clipping Mode', clippingModePreview, function()
    table.forEach(clippingModeTexts, function(item, key)
      if ui.selectable(item, self.mini.clippingMode == key) then
        self.mini.clippingMode = key
      end
    end)
  end)

  SpacingSeparator()

  -- Reset
  if ui.button('Reset to default') then
    table.forEach(self.defaultMini(), function(item, key)
      self.mini[key] = nil
    end)
  end
end

function MapMiniSettings:tabTrackRotation()
  -- hide
end

function MapMiniSettings:tabMapGenSizeConfig()
  self.mapGenStore.pxPerM = TooltipSlider('Image Size', self.mapGenStore.pxPerM, 0.05, 2.0, '%.2f px/m')
end

function MapMiniSettings:tabFreeroamTeleport()
  -- hide
end
